#include <bios.h>
#include "f2c.h"

extern "C" {
int sale_(char *qlin, ftnlen qlin_len);
int setcom_ (char* portch, ftnlen portch_len);
int getcom_ (char* portch, char* buf, ftnlen portch_len, ftnlen buf_len);
}

const unsigned int timeout   = 1 << 15;
const unsigned int xmtsempty = 1 << 14;
const unsigned int xmthempty = 1 << 13;
const unsigned int didbreak  = 1 << 12;
const unsigned int framerr   = 1 << 11;
const unsigned int parityerr = 1 << 10;
const unsigned int overrun   = 1 <<  9;
const unsigned int dataready = 1 <<  8;

const unsigned int datamask  = 0x7f;
const unsigned int receiverr = timeout + didbreak + framerr + parityerr + overrun;

int setcom_ (char* portch, ftnlen portch_len)
{
   int port = *portch - '1';
   if (port < 0 || port > 7)
      sale_("Serial port not in range 1..8", 29L);
   unsigned int stat = 
      _bios_serialcom(_COM_INIT, port, (_COM_CHR8|_COM_STOP2|_COM_NOPARITY|_COM_1200));
   if (stat & receiverr)
      sale_("Error setting up serial port", 28L);
   return 0;
}

int getcom_ (char* portch, char* buf, ftnlen portch_len, ftnlen buf_len)
{ 
   *buf     = 0;
   int port = *portch - '1';
   unsigned int status = _bios_serialcom(_COM_STATUS, port, 0);
   char ch = ' ';
   while (!(status & receiverr) && (ch != '\n') && (ch != '\r') && buf_len) {
      while (!(status & (dataready+receiverr))) {
         status = _bios_serialcom(_COM_STATUS, port, 0);
      }
      if (!(status & receiverr)) {
         status = _bios_serialcom(_COM_RECEIVE, port, 0);
         if (!(status & receiverr)) {
            ch = status & datamask;
            if (ch && (ch != '\n') && (ch != '\r') && buf_len) {
               *buf++ = ch;
               buf_len--;
            }
         }
      }
   }
   while (buf_len) {
      *buf++ = ' ';
      buf_len--;
   }
   if (!(status & receiverr)) {
      while (!(status & (dataready+receiverr))) {
         status = _bios_serialcom(_COM_STATUS, port, 0);
      }
      if (!(status & receiverr))
         status = _bios_serialcom(_COM_RECEIVE, port, 0);
   }
   if (status & receiverr)
      sale_("Error reading COM port", 22L);
   return 0;
}